/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package productcatalog.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import javax.sql.DataSource;
import productcatalog.Product;
import productcatalog.ProductManager;

/**
 *
 * @author petr
 */
public class ProductManagerImpl implements ProductManager {

    private DataSource dataSource;

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public void addProduct(Product product) throws SQLException {
        if (product.getId() != null) {
            throw new IllegalArgumentException("Product has already assigned id");
        }

        if (product.getName() == null || product.getDescription() == null) {
            throw new IllegalArgumentException("Product has null name or description");
        }
        
        Connection conn = null;
        PreparedStatement insertStatement = null;
        try {
            conn = dataSource.getConnection();
            insertStatement = conn.prepareStatement("INSERT INTO product " +
                    "(name, description, price) VALUES (?,?,?)",
                    Statement.RETURN_GENERATED_KEYS);
            insertStatement.setString(1, product.getName());
            insertStatement.setString(2, product.getDescription());
            insertStatement.setBigDecimal(3, product.getPrice());
            insertStatement.execute();

            ResultSet generatedKeys = insertStatement.getGeneratedKeys();
            if (generatedKeys.next()) {
                product.setId(generatedKeys.getLong(1));
            } else {
                throw new SQLException("Error when fetching generated key");
            }

        } finally {
            DBTools.closeQuietly(insertStatement);
            DBTools.closeQuietly(conn);
        }
    }

    public void deleteProduct(Product product) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void updateProduct(Product product) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public List<Product> getAllProducts() throws SQLException {
        Connection conn = null;
        PreparedStatement selectStatement = null;
        List<Product> result = new ArrayList<Product>();
        try {
            conn = dataSource.getConnection();
            selectStatement = conn.prepareStatement("SELECT * FROM product");

            ResultSet rsProducts = selectStatement.executeQuery();
            while (rsProducts.next()) {
                Product product = new Product();
                product.setId(rsProducts.getLong("id"));
                product.setName(rsProducts.getString("name"));
                product.setDescription(rsProducts.getString("description"));
                product.setPrice(rsProducts.getBigDecimal("price"));
                result.add(product);
            }
            return result;
        } finally {
            DBTools.closeQuietly(selectStatement);
            DBTools.closeQuietly(conn);
        }
    }

}
