/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package productcatalog.db;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import javax.sql.DataSource;
import org.apache.commons.dbcp.BasicDataSource;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import productcatalog.Product;
import productcatalog.ProductManager;

/**
 *
 * @author Petr Adamek
 */
public class ProductManagerImplTest {

    @Before
    public void setUp() throws IOException, SQLException {
        productManager = new ProductManagerImpl();

        DataSource ds = new DataSource() {

            public Connection getConnection() throws SQLException {
                return DriverManager.getConnection("jdbc:derby://localhost:1527/test",
                    "test","test");
            }

            public Connection getConnection(String username, String password) throws SQLException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public PrintWriter getLogWriter() throws SQLException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public void setLogWriter(PrintWriter out) throws SQLException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public void setLoginTimeout(int seconds) throws SQLException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public int getLoginTimeout() throws SQLException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public <T> T unwrap(Class<T> iface) throws SQLException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public boolean isWrapperFor(Class<?> iface) throws SQLException {
                throw new UnsupportedOperationException("Not supported yet.");
            }
        };

        productManager.setDataSource(ds);

        PreparedStatement st = ds.getConnection().prepareStatement(
                "DELETE FROM product");
        st.execute();

/*
        File dataDir = File.createTempFile("test", "db");
        dataDir.delete();
        //dataDir.mkdirs();

        BasicDataSource basicDataSource = new BasicDataSource();
        basicDataSource.setDriverClassName("org.apache.derby.jdbc.EmbeddedDriver");
        basicDataSource.setUrl("jdbc:derby:" +
                dataDir.getPath() + ";create=true");
        basicDataSource.getConnection(); */
    }

    @After
    public void tearDown() {
    }

    private ProductManagerImpl productManager;

    @Test
    public void testGetAllProducts() throws SQLException {
        List<Product> products = productManager.getAllProducts();
        assertTrue(products.isEmpty());
    }

    private Product newProduct(String name, String description, BigDecimal price) {
        Product product = new Product();
        product.setName(name);
        product.setDescription(description);
        product.setPrice(price);
        return product;
        
    }

    private void compareProductsDeeply(Product expected, Product actual) {
        assertEquals(expected.getName(),actual.getName());
        assertEquals(expected.getDescription(),actual.getDescription());
        assertEquals(expected.getPrice(),actual.getPrice());        
    }

    @Test
    public void testAddProduct() throws SQLException {
        Product productA = newProduct("Product A", "Some description",
                BigDecimal.valueOf(2852,2));
        assertNull(productA.getId());
        productManager.addProduct(productA);
        assertNotNull(productA.getId());

        Iterator<Product> i = productManager.getAllProducts().iterator();
        assertTrue(i.hasNext());
        Product returnedProduct = i.next();
        assertNotSame(productA, returnedProduct);
        compareProductsDeeply(productA, returnedProduct);
        assertFalse(i.hasNext());

        Product productB = newProduct("Product B", "Another description",
                BigDecimal.valueOf(2852,2));
        productManager.addProduct(productB);
        assertFalse(productA.getId().equals(productB.getId()));

        List<Product> products = productManager.getAllProducts();
        assertEquals(2, products.size());
        assertTrue(products.contains(productA));
        assertTrue(products.contains(productB));
    }

    @Test
    public void testAddProductWithExistingId() throws SQLException {
        try {
            Product product = newProduct("product", "desc", BigDecimal.ZERO);
            product.setId(Long.valueOf(1));
            productManager.addProduct(product);
            fail();
        } catch (IllegalArgumentException ex) {
            // this is ok
        }
    }

    @Test
    public void testAddProductWithWrongValues() throws SQLException {
        try {
            productManager.addProduct(newProduct(null, "desc", BigDecimal.TEN));
            fail();
        } catch (IllegalArgumentException ex) {
            // this is ok
        }
        try {
            productManager.addProduct(newProduct("product", null, BigDecimal.TEN));
            fail();
        } catch (IllegalArgumentException ex) {
            // this is ok
        }
    }

//    @Test
//    public void testDeleteProduct() {
//        fail("The test case is a prototype.");
//    }
//
//    @Test
//    public void testUpdateProduct() {
//        fail("The test case is a prototype.");
//    }

}